#define BOOST_TEST_MODULE weif_test
#include <boost/test/included/unit_test.hpp>
#include <boost/numeric/ublas/io.hpp>

#include "../../src/atmos.h"
#include "../../src/spectral_factory.h"
#include "../../src/weif.h"
#include "../../src/utils.h"

BOOST_AUTO_TEST_CASE( wind_factor ) {
	using atmos::weight::wind_factor;

	BOOST_CHECK_CLOSE( wind_factor(0.0),  1.0          , 2e-0);
	BOOST_CHECK_CLOSE( wind_factor(0.25), 0.9033059998 , 2e-0);
	BOOST_CHECK_CLOSE( wind_factor(0.5),  0.6766701410 , 2e-0);
	BOOST_CHECK_CLOSE( wind_factor(0.75), 0.4503268909 , 2e-0);
	BOOST_CHECK_CLOSE( wind_factor(1.0),  0.3092552256 , 2e-0);
	BOOST_CHECK_CLOSE( wind_factor(2.0),  0.1573429508 , 2e-0);
	BOOST_CHECK_CLOSE( wind_factor(4.0),  0.07923837253, 2e-0);
	BOOST_CHECK_CLOSE( wind_factor(5.0),  0.06346579361, 2e-0);
	BOOST_CHECK_CLOSE( wind_factor(10.0), 0.03179559908, 2e-0);
}

double smooth_limit( double height ) {
	return 2 * std::sqrt( height * 1000 * 5e-6 ) * 100;
}

BOOST_AUTO_TEST_CASE( weight_smoothed_test ) {
	using namespace utils::ublas;
	
	typedef spectral::vector_type vector_type;
	typedef vector< vector_type > vector_vector_type;
	typedef utils::ublas::cubic_spline< vector_type, vector_vector_type, vector_type, vector_vector_type > spline_type;
	std::pair< vector_type, vector_type > mass_reaction = spectral::load_reaction("./mass_maid.crv");
	std::pair< vector_type, vector_type > spectral_response = spectral::load_reaction("./a05.sp");
	std::pair< vector_type, vector_type > mass_response = spectral::photon_distribution( spectral_response.first, spectral_response.second, mass_reaction.first, mass_reaction.second );
	
	vector_type inner_diam(atmos::channels);
	vector_type outer_diam(atmos::channels);
	inner_diam(0)=0.0;outer_diam(0)=1.27;
	inner_diam(1)=1.3;outer_diam(1)=2.15;
	inner_diam(2)=2.2;outer_diam(2)=3.85;
	inner_diam(3)=3.9;outer_diam(3)=5.50;
	inner_diam = inner_diam * 14.5*0.1;
	outer_diam = outer_diam * 14.5*0.1;

	atmos::weight::mass_variableexposure_weight wf( mass_response.first, mass_response.second, inner_diam, outer_diam );
	
	vector_type height_grid = vector_generator< atmos::lg_generator >( atmos::lg_generator(0.0, 0.04, 0.25), 50 );
	vector_type base_smooth_grid = vector_generator< atmos::lg_generator >( atmos::lg_generator(0.0, 0.1, 0.8), 8);

	std::cerr << base_smooth_grid << std::endl;
	
	for( vector_type::size_type h = 1; h < height_grid.size(); ++h ) {
		std::cout << "#" << height_grid(h) << " ";
		
		vector_type smooth_grid = smooth_limit(height_grid(h))*base_smooth_grid;
		vector_vector_type variableexposure_values = project( smooth_grid, wf(height_grid(h)) );
		spline_type spl( smooth_grid,
				variableexposure_values,
				vector_type(zero_vector<double>(variableexposure_values(0).size())),
				vector_type(-variableexposure_values(variableexposure_values.size()-1)/smooth_grid(smooth_grid.size()-1)) );
		
		double last_node = spl.first_data()(spl.first_data().size()-1);
		for( double smooth = 0.0; smooth < 200.0; smooth += 0.5 ) {
			if( last_node > smooth ) {
				boost::tuple< vector_type, vector_type, vector_type > f = spl(smooth,true);
				std::cout << " " << f.get<0>()(3);
				continue;
			}
			vector_type last_value = spl.second_data()(spl.second_data().size()-1);
			std::cout << " " << last_value(3)*last_node/smooth;
			//std::cout << " " << last_value(0)*last_node/(-smooth*smooth);
		}
		std::cout << std::endl;
	}
}

BOOST_AUTO_TEST_CASE( weight_longexposure_test ) {
	using namespace utils::ublas;
	
	typedef spectral::vector_type vector_type;
	std::pair< vector_type, vector_type > mass_reaction = spectral::load_reaction("./mass_maid.crv");
	std::pair< vector_type, vector_type > spectral_response = spectral::load_reaction("./a05.sp");
	std::pair< vector_type, vector_type > mass_response = spectral::photon_distribution( spectral_response.first, spectral_response.second, mass_reaction.first, mass_reaction.second );
	
	vector_type inner_diam(atmos::channels);
	vector_type outer_diam(atmos::channels);
	inner_diam(0)=0.0;outer_diam(0)=1.27;
	inner_diam(1)=1.3;outer_diam(1)=2.15;
	inner_diam(2)=2.2;outer_diam(2)=3.85;
	inner_diam(3)=3.9;outer_diam(3)=5.50;
	inner_diam = inner_diam * 14.5*0.1;
	outer_diam = outer_diam * 14.5*0.1;

	atmos::weight::mass_longexposure_weight wf( mass_response.first, mass_response.second, inner_diam, outer_diam );
	
	vector_type height_grid = vector_generator< atmos::lg_generator >( atmos::lg_generator(0.0, 0.04, 0.25), 50 );
	
	for( vector_type::size_type h = 0; h < height_grid.size(); ++h ) {
		std::cout << wf(height_grid(h))(0) << "\t";
	}
	std::cout << std::endl;
}

BOOST_AUTO_TEST_CASE( weight_shortexposure_test ) {
	using namespace utils::ublas;
	
	typedef spectral::vector_type vector_type;
	std::pair< vector_type, vector_type > mass_reaction = spectral::load_reaction("./mass_maid.crv");
	std::pair< vector_type, vector_type > spectral_response = spectral::load_reaction("./a05.sp");
	std::pair< vector_type, vector_type > mass_response = spectral::photon_distribution( spectral_response.first, spectral_response.second, mass_reaction.first, mass_reaction.second );
	
	vector_type inner_diam(atmos::channels);
	vector_type outer_diam(atmos::channels);
	inner_diam(0)=0.0;outer_diam(0)=1.27;
	inner_diam(1)=1.3;outer_diam(1)=2.15;
	inner_diam(2)=2.2;outer_diam(2)=3.85;
	inner_diam(3)=3.9;outer_diam(3)=5.50;
	inner_diam = inner_diam * 14.5*0.1;
	outer_diam = outer_diam * 14.5*0.1;

	atmos::weight::mass_shortexposure_weight wf( mass_response.first, mass_response.second, inner_diam, outer_diam );
	
	vector_type height_grid = vector_generator< atmos::lg_generator >( atmos::lg_generator(0.0, 0.04, 0.25), 50 );
	
	for( vector_type::size_type h = 0; h < height_grid.size(); ++h ) {
		std::cout << wf(height_grid(h))(0) << "\t";
	}
	std::cout << std::endl;
}
