#ifndef _TRM_DRIVER_H
#define _TRM_DRIVER_H

#include "debug.h"
#include "traits.h"

namespace trm {
template< class T, template<class C> class InitialDeltaStrategy,
	template<class C> class CoreIterationStrategy,
	template<class C> class UpdateDeltaStrategy,
	template<class C> class FunctionDecorator,
	class StopStrategy > class basic_driver {
public:
	typedef basic_driver<T,InitialDeltaStrategy,CoreIterationStrategy,UpdateDeltaStrategy,FunctionDecorator,StopStrategy> self_type;
	
	typedef InitialDeltaStrategy<self_type>  initial_delta_strategy_type;
	typedef CoreIterationStrategy<self_type> core_iteration_strategy_type;
	typedef UpdateDeltaStrategy<self_type>   update_delta_strategy_type;
	typedef FunctionDecorator<self_type>     function_decorator_type;
	typedef StopStrategy                     stop_strategy_type;
	
	typedef typename trm::type_traits<T>::value_type    value_type;
	typedef typename trm::type_traits<T>::vector_type   vector_type;
	typedef typename trm::type_traits<T>::matrix_type   matrix_type;
	typedef typename trm::type_traits<T>::function_type function_type;
	typedef typename trm::type_traits<T>::grad_type     grad_type;
	typedef typename trm::type_traits<T>::hesse_type    hesse_type;
public:
	basic_driver( const vector_type& x, const typename function_decorator_type::functor_type& f, const stop_strategy_type& stop_strategy_  ):
		initial_x_(x),
		function_decorator_(f),
		core_iteration_strategy_(*this),
		stop_strategy_(stop_strategy_) { }
	
	const vector_type& initial_x() const { return initial_x_; }
	boost::tuple< function_type, grad_type, hesse_type > evaluate_function( const vector_type& x ) {
		return function_decorator_.evaluate_function(x);
	}
	vector_type update_x( const vector_type& new_x, const vector_type& step ) const {
		return function_decorator_.update_x(new_x,step);
	}
	
	boost::tuple< stop_strategy_type, function_type, vector_type > minimize() {
		core_iteration_strategy_.update_delta(initial_delta_strategy_.initial_delta(*this));
		
		TRM_DEBUG_PRINT_BEGIN();
		TRM_DEBUG_PRINT_INFO( core_iteration_strategy_.delta(), core_iteration_strategy_.step_norm(), core_iteration_strategy_.step() , core_iteration_strategy_.rho_ratio() );
		TRM_DEBUG_PRINT_FUNCTION( core_iteration_strategy_.x(), core_iteration_strategy_.function(), core_iteration_strategy_.grad(), core_iteration_strategy_.hesse() );

		while( true ) {
			core_iteration_strategy_.iterate();
			
			TRM_DEBUG_PRINT_INFO( core_iteration_strategy_.delta(), core_iteration_strategy_.step_norm(), core_iteration_strategy_.step() , core_iteration_strategy_.rho_ratio() );

			// FIXME: we need more general function update_delta_strategy_.need_update( rho_ratio )
			if( core_iteration_strategy_.rho_ratio() > update_delta_strategy_.rho_ratio_threshold() ) {
				core_iteration_strategy_.update_x( function_decorator_.update_x(core_iteration_strategy_.x(), core_iteration_strategy_.step()) );
			}

			TRM_DEBUG_PRINT_FUNCTION( core_iteration_strategy_.x(), core_iteration_strategy_.function(), core_iteration_strategy_.grad(), core_iteration_strategy_.hesse() );

			if( stop_strategy_.check_iternum() ) break;
			if( core_iteration_strategy_.check_stop() ) break;
			core_iteration_strategy_.update_delta(update_delta_strategy_.update_delta(*this));
		}
		
		TRM_DEBUG_PRINT_INFO( core_iteration_strategy_.delta(), core_iteration_strategy_.step_norm(), core_iteration_strategy_.step() , core_iteration_strategy_.rho_ratio() );
		TRM_DEBUG_PRINT_FUNCTION( core_iteration_strategy_.x(), core_iteration_strategy_.function(), core_iteration_strategy_.grad(), core_iteration_strategy_.hesse() );
		TRM_DEBUG_PRINT_END(stop_strategy_);
		
		return boost::make_tuple( stop_strategy_, core_iteration_strategy_.function(), core_iteration_strategy_.x() );
	}
	
	value_type last_step_norm() const { return core_iteration_strategy_.step_norm(); }
	value_type last_rho_ratio() const { return core_iteration_strategy_.rho_ratio(); }
	value_type last_delta() const { return core_iteration_strategy_.delta(); }
	
	stop_strategy_type& stop_strategy() { return stop_strategy_; }
private:
	vector_type initial_x_;

	function_decorator_type      function_decorator_;
	initial_delta_strategy_type  initial_delta_strategy_;
	core_iteration_strategy_type core_iteration_strategy_;
	update_delta_strategy_type   update_delta_strategy_;
	stop_strategy_type           stop_strategy_;
};
}

#endif // _TRM_DRIVER_H
