#ifndef _TRM_FUNCTOR_H
#define _TRM_FUNCTOR_H

#include <boost/function.hpp>

namespace trm {
namespace functor {
	template<class Context> struct basic {
		typedef typename Context::value_type value_type;
		typedef typename Context::vector_type vector_type;
		typedef typename Context::function_type function_type;
		typedef typename Context::grad_type  grad_type;
		typedef typename Context::hesse_type hesse_type;
		
		typedef typename boost::tuple<function_type,grad_type,hesse_type> return_type;
		typedef boost::function1< boost::tuple<function_type,grad_type,hesse_type>, vector_type > functor_type;
		
		basic( const functor_type& function ): function_(function) {}
		
		virtual return_type evaluate_function( const vector_type& x ) {
			return function_(x);
		}
		virtual vector_type update_x( const vector_type& old_x, const vector_type& step ) const {
			return old_x + step;
		}
	private:
		functor_type function_;
	};
	
	template<class Context> struct least_squares {
		typedef typename Context::value_type value_type;
		typedef typename Context::vector_type vector_type;
		typedef typename Context::matrix_type matrix_type;
		typedef typename Context::function_type function_type;
		typedef typename Context::grad_type  grad_type;
		typedef typename Context::hesse_type hesse_type;
		
		typedef typename boost::tuple<function_type,grad_type,hesse_type> return_type;
		typedef boost::function1< boost::tuple<vector_type,matrix_type,hesse_type>, vector_type> functor_type;
		
		least_squares( const functor_type& function ): function_(function) {}
		
		virtual return_type evaluate_function( const vector_type& x ) {
			using namespace boost::numeric::ublas;
			
			vector_type residual;
			matrix_type jacobian;
			hesse_type  hesse;
			boost::tie(residual,jacobian,hesse) = function_(x);
			return boost::make_tuple( 0.5*inner_prod(residual,residual), prod(trans(jacobian),residual), prod(trans(jacobian),jacobian)+hesse );
		}
		virtual vector_type update_x( const vector_type& old_x, const vector_type& step ) const {
			return old_x + step;
		}
	private:
		functor_type function_;
	};
}
}

#endif // _TRM_FUNCTOR_H
