#define BOOST_TEST_MODULE steihaug_toint
#include <boost/test/included/unit_test.hpp>

#include "gauss.h"
#include "rosenbrock.h"
#include "quadratic.h"

#include <steihaug_toint.h>

template<class T,class F> void test_steihaug_toint( const ublas::vector<T>& initial_x, F function, T result, T abstol = 1e-6 ) {
	T function_min = 0.;
	boost::tie( boost::tuples::ignore, function_min, boost::tuples::ignore ) = trm::steihaug_toint(initial_x,function,trm::stop::basic<T>());

	BOOST_TEST_MESSAGE( "Function minimum: " << function_min );
	if( result != 0.0 ) {
		BOOST_CHECK_CLOSE( result, function_min, abstol );
	} else {
		BOOST_CHECK_SMALL( function_min, abstol );
	}
}

template<class T,class F> void test_steihaug_toint_nonnegative( const ublas::vector<T>& initial_x, F function, T result, T abstol = 1e-6 ) {
	T function_min = 0.;
	boost::tie( boost::tuples::ignore, function_min, boost::tuples::ignore ) = trm::steihaug_toint_nonnegative(initial_x,function,trm::stop::basic<T>());

	BOOST_TEST_MESSAGE( "Function minimum: " << function_min );
	if( result != 0.0 ) {
		BOOST_CHECK_CLOSE( result, function_min, abstol );
	} else {
		BOOST_CHECK_SMALL( function_min, abstol );
	}
}

template<class T,class F> void test_steihaug_toint_least_squares( const ublas::vector<T>& initial_x, F function, T result, T abstol = 1e-6 ) {
	T function_min = 0.;
	boost::tie( boost::tuples::ignore, function_min, boost::tuples::ignore ) = trm::steihaug_toint_least_squares(initial_x,function,trm::stop::basic<T>());

	BOOST_TEST_MESSAGE( "Function minimum: " << function_min );
	if( result != 0.0 ) {
		BOOST_CHECK_CLOSE( result, function_min, abstol );
	} else {
		BOOST_CHECK_SMALL( function_min, abstol );
	}
}

template<class T,class F> void test_steihaug_toint_nonnegative_least_squares( const ublas::vector<T>& initial_x, F function, T result, T abstol = 1e-6 ) {
	T function_min = 0.;
	boost::tie( boost::tuples::ignore, function_min, boost::tuples::ignore ) = trm::steihaug_toint_nonnegative_least_squares(initial_x,function,trm::stop::basic<T>());

	BOOST_TEST_MESSAGE( "Function minimum: " << function_min );
	if( result != 0.0 ) {
		BOOST_CHECK_CLOSE( result, function_min, abstol );
	} else {
		BOOST_CHECK_SMALL( function_min, abstol );
	}
}

BOOST_AUTO_TEST_CASE( gauss_test ) {
	ublas::vector<double> x0(5);
	for( int i =0;i<x0.size();i++){
		x0(i)=1;
	}
	ublas::vector<double> center = 2 * x0;
	
	gauss g(center);
        test_steihaug_toint( x0,g , 0.0 );
}

BOOST_AUTO_TEST_CASE( rosenbrock_test ) {
	ublas::vector<double> x0(2);
	for( int i =0;i<x0.size();i++){
		x0(i)=1+i*2;
	}
	rosenbrock g;
        test_steihaug_toint( x0,g , 0.0 );
}

BOOST_AUTO_TEST_CASE( quadratic_test ) {
	ublas::vector<double> x0(2);
	x0(0)=1;
	x0(1)=1;
	ublas::vector<double> center;
	
	center = 2*x0;
	quadratic q1(center);
        test_steihaug_toint( x0,q1, 0.0 );
	
	center = -center;
	quadratic q2(center);
        test_steihaug_toint( x0,q2, 0.0 );

	center(0)=-center(0);
	quadratic q3(center);
        test_steihaug_toint( x0,q3, 0.0 );
	
	center = -center;
	quadratic q4(center);
        test_steihaug_toint( x0,q4, 0.0 );
}

BOOST_AUTO_TEST_CASE( gauss_test_nonnegative ) {
	ublas::vector<double> x0(5);
	for( int i =0;i<x0.size();i++){
		x0(i)=1;
	}
	ublas::vector<double> center = -x0;
	
	gauss g(center);
        test_steihaug_toint_nonnegative( x0,g , 3.1606027941427883 );
}

BOOST_AUTO_TEST_CASE( rosenbrock_test_nonnegative ) {
	ublas::vector<double> x0(2);
	for( int i =0;i<x0.size();i++){
		x0(i)=1+i*2;
	}
	rosenbrock g;
        test_steihaug_toint_nonnegative( x0,g , 0.0 );
}

BOOST_AUTO_TEST_CASE( quadratic_test_nonnegative ) {
	ublas::vector<double> x0(2);
	x0(0)=1;
	x0(1)=1;
	ublas::vector<double> center;
	
	center = 2*x0;
	quadratic q1(center);
        test_steihaug_toint_nonnegative( x0,q1, 0.0 );
	
	center = -center;
	quadratic q2(center);
        test_steihaug_toint_nonnegative( x0,q2, 4*2.0 );

	center(0)=-center(0);
	quadratic q3(center);
        test_steihaug_toint_nonnegative( x0,q3, 4*1.0 );
	
	center = -center;
	quadratic q4(center);
        test_steihaug_toint_nonnegative( x0,q4, 4*1.0 );
}

BOOST_AUTO_TEST_CASE( gauss_test_least_squares ) {
	ublas::vector<double> x0(5);
	for( int i =0;i<x0.size();i++){
		x0(i)=1;
	}
	ublas::vector<double> center = 2 * x0;
	
	gauss_least_squares g(center);
        test_steihaug_toint_least_squares( x0,g , 0.0 );
}

BOOST_AUTO_TEST_CASE( gauss_test_nonnegative_least_squares ) {
	ublas::vector<double> x0(5);
	for( int i =0;i<x0.size();i++){
		x0(i)=1;
	}
	ublas::vector<double> center = 2 * x0;
	
	gauss_least_squares g(center);
        test_steihaug_toint_nonnegative_least_squares( x0,g , 0.0 );
}